"""Define endpoints for interacting with devices (phones, etc. with Notion)."""
from typing import Callable


class Device:  # pylint: disable=too-few-public-methods
    """Define an object to interact with all endpoints."""

    def __init__(self, request: Callable) -> None:
        """Initialize."""
        self._request = request

    async def async_all(self) -> list:
        """Get all devices."""
        resp = await self._request("get", "devices")
        return resp["devices"]

    async def async_create(self, attributes: dict) -> dict:
        """Create a device with a specific attribute payload."""
        resp = await self._request("post", "devices", json={"devices": attributes})
        return resp["devices"]

    async def async_delete(self, device_id: int) -> None:
        """Delete a device by ID."""
        await self._request("delete", "devices/{0}".format(device_id))

    async def async_get(self, device_id: int) -> dict:
        """Get a device by ID."""
        resp = await self._request("get", "devices/{0}".format(device_id))
        return resp["devices"]



"""Define a base client for interacting with Notion."""
from typing import Optional

from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientError

from .bridge import Bridge
from .device import Device
from .errors import RequestError
from .sensor import Sensor
from .system import System
from .task import Task

API_BASE = "https://api.getnotion.com/api"


class Client:  # pylint: disable=too-few-public-methods
    """Define the API object."""

    def __init__(self, session: ClientSession) -> None:
        """Initialize."""
        self._session = session
        self._token = None  # type: Optional[str]

        self.bridge = Bridge(self._request)
        self.device = Device(self._request)
        self.sensor = Sensor(self._request)
        self.system = System(self._request)
        self.task = Task(self._request)

    async def _request(
        self,
        method: str,
        endpoint: str,
        *,
        headers: dict = None,
        params: dict = None,
        json: dict = None
    ) -> dict:
        """Make a request the API.com."""
        url = "{0}/{1}".format(API_BASE, endpoint)

        if not headers:
            headers = {}

        if self._token:
            headers["Authorization"] = "Token token={0}".format(self._token)

        async with self._session.request(
            method, url, headers=headers, params=params, json=json
        ) as resp:
            data = await resp.json(content_type=None)
            try:
                resp.raise_for_status()
                return data
            except ClientError:
                raise RequestError(data["errors"][0]["title"])

    async def async_authenticate(self, email: str, password: str) -> None:
        """Authenticate the user and retrieve an authentication token."""
        auth_response = await self._request(
            "post",
            "users/sign_in",
            json={"sessions": {"email": email, "password": password}},
        )

        self._token = auth_response["session"]["authentication_token"]


async def async_get_client(email: str, password: str, session: ClientSession) -> Client:
    """Return an authenticated API object."""
    client = Client(session)
    await client.async_authenticate(email, password)
    return client



"""Define endpoints for interacting with sensors."""
from typing import Callable


class Sensor:  # pylint: disable=too-few-public-methods
    """Define an object to interact with all endpoints."""

    def __init__(self, request: Callable) -> None:
        """Initialize."""
        self._request = request

    async def async_all(self) -> list:
        """Get all sensors."""
        resp = await self._request("get", "sensors")
        return resp["sensors"]

    async def async_create(self, attributes: dict) -> dict:
        """Create a sensor with a specific attribute payload."""
        resp = await self._request("post", "sensors", json={"sensors": attributes})
        return resp["sensors"]

    async def async_delete(self, sensor_id: int) -> None:
        """Delete a sensor by ID."""
        await self._request("delete", "sensors/{0}".format(sensor_id))

    async def async_get(self, sensor_id: int) -> dict:
        """Get a sensor by ID."""
        resp = await self._request("get", "sensors/{0}".format(sensor_id))
        return resp["sensors"]

    async def async_update(self, sensor_id: int, new_attributes: dict) -> dict:
        """Update a sensor with a specific attribute payload."""
        resp = await self._request(
            "put", "sensors/{0}".format(sensor_id), json={"sensors": new_attributes}
        )
        return resp["sensors"]



"""Define fixtures for tasks."""
import pytest


@pytest.fixture()
def task_all_json():
    """Define a response to GET /tasks."""
    return {
        "tasks": [
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "missing",
                "sensor_data": [],
                "status": {
                    "value": "missing",
                    "received_at": "2019-04-30T03:57:41.716Z",
                },
                "created_at": "2019-04-30T01:56:46.004Z",
                "updated_at": "2019-04-30T03:57:42.004Z",
                "sensor_id": 132470,
                "model_version": "2.0",
                "configuration": {},
                "links": {"sensor": 123456},
            },
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "leak",
                "sensor_data": [],
                "status": {
                    "value": "no_leak",
                    "received_at": "2019-04-30T01:57:31.206Z",
                },
                "created_at": "2019-04-30T01:57:31.206Z",
                "updated_at": "2019-04-30T01:57:31.268Z",
                "sensor_id": 132470,
                "model_version": "2.1",
                "configuration": {},
                "links": {"sensor": 123456},
            },
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "alarm",
                "sensor_data": [],
                "status": {
                    "value": "no_alarm",
                    "received_at": "2019-04-30T01:57:30.972Z",
                },
                "created_at": "2019-04-30T01:57:30.971Z",
                "updated_at": "2019-04-30T01:57:31.046Z",
                "sensor_id": 132470,
                "model_version": "3.0",
                "configuration": {},
                "links": {"sensor": 123456},
            },
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "low_battery",
                "sensor_data": [],
                "status": {
                    "value": "battery_good",
                    "received_at": "2019-04-30T01:56:45.955Z",
                },
                "created_at": "2019-04-30T01:56:45.955Z",
                "updated_at": "2019-04-30T01:56:45.974Z",
                "sensor_id": 132470,
                "model_version": "2.0",
                "configuration": {},
                "links": {"sensor": 123456},
            },
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "missing",
                "sensor_data": [],
                "status": {
                    "value": "missing",
                    "received_at": "2019-04-30T03:57:41.716Z",
                },
                "created_at": "2019-04-30T01:45:14.370Z",
                "updated_at": "2019-04-30T03:57:41.981Z",
                "sensor_id": 132462,
                "model_version": "2.0",
                "configuration": {},
                "links": {"sensor": 123456},
            },
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "temperature",
                "sensor_data": [],
                "status": {
                    "value": "20.2838134765625",
                    "received_at": "2019-04-30T03:43:34.765Z",
                },
                "created_at": "2019-04-30T01:45:51.868Z",
                "updated_at": "2019-04-30T01:45:51.930Z",
                "sensor_id": 132462,
                "model_version": "2.1",
                "configuration": {"lower": 15.56, "upper": 29.44, "offset": 0.0},
                "links": {"sensor": 123456},
            },
            {
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "task_type": "low_battery",
                "sensor_data": [],
                "status": {
                    "value": "battery_good",
                    "received_at": "2019-04-30T01:45:14.221Z",
                },
                "created_at": "2019-04-30T01:45:14.221Z",
                "updated_at": "2019-04-30T01:45:14.230Z",
                "sensor_id": 132462,
                "model_version": "2.0",
                "configuration": {},
                "links": {"sensor": 123456},
            },
        ]
    }


@pytest.fixture()
def task_create_json():
    """Define a response to POST /sensors/:id/tasks."""
    return {
        "tasks": {
            "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "task_type": "missing",
            "sensor_data": [],
            "status": {"value": "missing", "received_at": "2019-04-30T03:57:41.716Z"},
            "created_at": "2019-04-30T01:56:46.004Z",
            "updated_at": "2019-04-30T03:57:42.004Z",
            "sensor_id": 132470,
            "model_version": "2.0",
            "configuration": {},
            "links": {"sensor": 123456},
        }
    }


@pytest.fixture()
def task_get_json():
    """Define a response to GET /tasks/:id."""
    return {
        "tasks": {
            "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "task_type": "missing",
            "sensor_data": [],
            "status": {"value": "missing", "received_at": "2019-04-30T03:57:41.716Z"},
            "created_at": "2019-04-30T01:56:46.004Z",
            "updated_at": "2019-04-30T03:57:42.004Z",
            "sensor_id": 132470,
            "model_version": "2.0",
            "configuration": {},
            "links": {"sensor": 123456},
        }
    }


@pytest.fixture()
def task_history_json():
    """Define a response to GET /tasks/:id/data."""
    return {
        "task": {
            "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "task_type": "temperature",
            "data": [
                {"time": "2019-06-17T04:09:55.786Z", "value": "23"},
                {"time": "2019-06-17T04:19:46.950Z", "value": "23"},
                {"time": "2019-06-17T04:29:37.572Z", "value": "22"},
            ],
        }
    }



"""Define fixtures for sensors."""
import pytest

from ..const import TEST_EMAIL


@pytest.fixture()
def sensor_all_json():
    """Define a response to GET /sensors."""
    return {
        "sensors": [
            {
                "id": 123456,
                "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "user": {"id": 12345, "email": TEST_EMAIL},
                "bridge": {"id": 12345, "hardware_id": "0x1234567890abcdef"},
                "last_bridge_hardware_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "name": "Bathroom Sensor",
                "location_id": 123456,
                "system_id": 12345,
                "hardware_id": "0x1234567890abcdef",
                "firmware_version": "1.1.2",
                "hardware_revision": 5,
                "device_key": "0x1234567890abcdef",
                "encryption_key": True,
                "installed_at": "2019-04-30T01:57:34.443Z",
                "calibrated_at": "2019-04-30T01:57:35.651Z",
                "last_reported_at": "2019-04-30T02:20:04.821Z",
                "missing_at": None,
                "updated_at": "2019-04-30T01:57:36.129Z",
                "created_at": "2019-04-30T01:56:45.932Z",
                "signal_strength": 5,
                "links": {"location": 123456},
                "lqi": 0,
                "rssi": -46,
                "surface_type": None,
            },
            {
                "id": 132462,
                "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "user": {"id": 12345, "email": TEST_EMAIL},
                "bridge": {"id": 12345, "hardware_id": "0x1234567890abcdef"},
                "last_bridge_hardware_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "name": "Living Room Sensor",
                "location_id": 123456,
                "system_id": 12345,
                "hardware_id": "0x1234567890abcdef",
                "firmware_version": "1.1.2",
                "hardware_revision": 5,
                "device_key": "0x1234567890abcdef",
                "encryption_key": True,
                "installed_at": "2019-04-30T01:45:56.169Z",
                "calibrated_at": "2019-04-30T01:46:06.256Z",
                "last_reported_at": "2019-04-30T02:20:04.829Z",
                "missing_at": None,
                "updated_at": "2019-04-30T01:46:07.717Z",
                "created_at": "2019-04-30T01:45:14.148Z",
                "signal_strength": 5,
                "links": {"location": 123456},
                "lqi": 0,
                "rssi": -30,
                "surface_type": None,
            },
        ]
    }


@pytest.fixture()
def sensor_create_json():
    """Define a response to POST /sensors."""
    return {
        "sensors": {
            "id": 123456,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "user": {"id": 12345, "email": TEST_EMAIL},
            "bridge": {"id": 12345, "hardware_id": "0x1234567890abcdef"},
            "last_bridge_hardware_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "name": "New Sensor",
            "location_id": 123456,
            "system_id": 12345,
            "hardware_id": "0x1234567890abcdef",
            "firmware_version": "1.1.2",
            "hardware_revision": 5,
            "device_key": "0x1234567890abcdef",
            "encryption_key": True,
            "installed_at": "2019-04-30T01:57:34.443Z",
            "calibrated_at": "2019-04-30T01:57:35.651Z",
            "last_reported_at": "2019-04-30T02:20:04.821Z",
            "missing_at": None,
            "updated_at": "2019-04-30T01:57:36.129Z",
            "created_at": "2019-04-30T01:56:45.932Z",
            "signal_strength": 5,
            "links": {"location": 123456},
            "lqi": 0,
            "rssi": -46,
            "surface_type": None,
        }
    }


@pytest.fixture()
def sensor_get_json():
    """Define a response to GET /sensors/:id."""
    return {
        "sensors": {
            "id": 123456,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "user": {"id": 12345, "email": TEST_EMAIL},
            "bridge": {"id": 12345, "hardware_id": "0x1234567890abcdef"},
            "last_bridge_hardware_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "name": "Bathroom Sensor",
            "location_id": 123456,
            "system_id": 12345,
            "hardware_id": "0x1234567890abcdef",
            "firmware_version": "1.1.2",
            "hardware_revision": 5,
            "device_key": "0x1234567890abcdef",
            "encryption_key": True,
            "installed_at": "2019-04-30T01:57:34.443Z",
            "calibrated_at": "2019-04-30T01:57:35.651Z",
            "last_reported_at": "2019-04-30T02:20:04.821Z",
            "missing_at": None,
            "updated_at": "2019-04-30T01:57:36.129Z",
            "created_at": "2019-04-30T01:56:45.932Z",
            "signal_strength": 5,
            "links": {"location": 123456},
            "lqi": 0,
            "rssi": -46,
            "surface_type": None,
        }
    }


@pytest.fixture()
def sensor_update_json():
    """Define a response to PUT /sensors/:id."""
    return {
        "sensors": {
            "id": 123456,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "user": {"id": 12345, "email": TEST_EMAIL},
            "bridge": {"id": 12345, "hardware_id": "0x1234567890abcdef"},
            "last_bridge_hardware_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "name": "Updated Sensor Name",
            "location_id": 123456,
            "system_id": 12345,
            "hardware_id": "0x1234567890abcdef",
            "firmware_version": "1.1.2",
            "hardware_revision": 5,
            "device_key": "0x1234567890abcdef",
            "encryption_key": True,
            "installed_at": "2019-04-30T01:57:34.443Z",
            "calibrated_at": "2019-04-30T01:57:35.651Z",
            "last_reported_at": "2019-04-30T02:20:04.821Z",
            "missing_at": None,
            "updated_at": "2019-04-30T01:57:36.129Z",
            "created_at": "2019-04-30T01:56:45.932Z",
            "signal_strength": 5,
            "links": {"location": 123456},
            "lqi": 0,
            "rssi": -46,
            "surface_type": None,
        }
    }
